﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using Abt.Controls.SciChart;
using Microsoft.Win32;

namespace ScreenshotsAndXpsExport
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void CopyToClipboardClick(object sender, RoutedEventArgs e)
        {
            // See SciChartSurfaceExtensions where ExportBitmapToClipboard is defined
            this.customUserControl.SciChartSurface.ExportBitmapToClipboard();

            MessageBox.Show("Copied to Clipboard!");
        }

        private void SaveAsPngClick(object sender, RoutedEventArgs e)
        {
            var dialog = new SaveFileDialog();
            dialog.DefaultExt = "png";
            dialog.AddExtension = true;
            dialog.Filter = "Png Files|*.png";
            dialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
            if (dialog.ShowDialog() == true)
            {
                // See SciChartSurfaceExtensions where ExportBitmapToFile is defined
                this.customUserControl.SciChartSurface.ExportBitmapToFile(dialog.FileName);
                Process.Start(dialog.FileName);
            }
        }

        private void PrintToXpsClick(object sender, RoutedEventArgs e)
        {
            var dialog = new PrintDialog();
            if (dialog.ShowDialog() == true)
            {
                var size = new Size(dialog.PrintableAreaWidth, dialog.PrintableAreaWidth*3/4);

                var scs = CreateSciChartSurfaceWithoutShowingIt(size);

                // And print. This works particularly well to XPS! 
                Action printAction = () => dialog.PrintVisual(scs, "Programmer Awesomeness");
                Dispatcher.BeginInvoke(printAction);
            }
        }

        /// <summary>
        /// This method demonstrates how we can render to bitmap or print a SciChartSurface which is never shown, 
        /// e.g. if you wanted to create a report on a server, or wanted to grab a screenshot at a different resolution
        /// to the currently shown size of the chart
        /// </summary>
        /// <param name="size"></param>
        /// <returns></returns>
        private Visual CreateSciChartSurfaceWithoutShowingIt(Size size)
        {
            // Create a fresh ChartView, this contains the ViewModel (declared in XAML) and data
            var control = new ChartView();
            var scs = control.SciChartSurface;

            // RenderPriority.Immediate is required to print or render to bitmap any SciChartSurface that has not been 
            // added to the Visual Tree. This ensures that re-draw events are processed synchronously
            //
            // Note RenderPriority.Immediate should never be used for real-time charts!! 
            scs.RenderPriority = RenderPriority.Immediate;

            // ApplyTemplate is required on the newly created chart to ensure the theme is applied
            scs.ApplyTemplate();

            // Set the width/height explicitly based on print area
            scs.Width = size.Width;
            scs.Height = size.Height;

            // Force the Loaded event for the SciChartSurface
            scs.OnLoad();

            // Now measure & arrange
            scs.Measure(new Size(scs.Width, scs.Height));
            scs.Arrange(new Rect(new Point(0, 0), scs.DesiredSize));
            scs.UpdateLayout();

            // Finally trigger a redraw
            scs.InvalidateElement();

            return scs;
        }

        private void CustomUserControl_OnMouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            // See SciChartSurfaceExtensions where ExportBitmapToClipboard is defined
            this.customUserControl.SciChartSurface.ExportBitmapToClipboard();

            MessageBox.Show("Copied to Clipboard!");
        }
    }
}
